module Microservice
  module ApplicationHelper

    def application_name
      Microservice::Settings.application_name
    end

    def render_flash_messages
      s = ''
      flash.each do |k, v|
        case k
        when 'alert'
          message_css = 'warning'
          icon_css = 'warning'
        else
          message_css = 'success'
          icon_css = 'info'
        end
        s << "<div class=\"ui icon #{message_css} message\">"
        s << '<i class="close icon"></i>'
        s << "<i class=\"#{icon_css} icon\"></i>"
        s << '<div class=\"content\"></div>'
        s << "<div class=\"header\">#{v}</div>"
        s << '</div>'
      end
      s.html_safe
    end

    def html_title(*args)
      if args.empty?
        title = [application_name]
        title.concat(@html_title || [])
        title.reject(&:blank?).join(' - ')
      else
        @html_title ||= []
        @html_title += args
      end
    end

    def title(*args)
      strings = args.map do |arg|
        if arg.is_a?(Array) && arg.size >= 2
          link_to(*arg)
        else
          h(arg.to_s)
        end
      end

      html_title(args.reverse.map {|s| (s.is_a?(Array) ? s.first : s).to_s})

      content_tag('h1', strings.join(' &#187; ').html_safe, class: 'ui header')
    end

    def favicon_path
      image_path('microservice/favicon.ico')
    end

    def html_title_from_entity
      if @entity
        case action_name
        when 'show'
          html_title @entity.heading_show
        when 'new', 'create'
          html_title @entity.heading_new
        when 'edit', 'update'
          html_title @entity.heading_edit
        else
          html_title
        end
      elsif controller.respond_to?(:entity_class) && controller.entity_class
        html_title controller.entity_class.heading_index
      else
        html_title
      end
    end

    def format_date(date)
      return nil unless date

      ::I18n.l(date.to_date, {format: '%d.%m.%Y'})
    end

    def format_time(time, include_date = true)
      return nil unless time
      options = {}
      options[:format] = '%H:%M'
      time = time.to_time if time.is_a?(String)
      local = (time.utc? ? time.localtime : time)
      (include_date ? "#{format_date(local)} " : '') + ::I18n.l(local, options)
    end

    def css_active_menu_item(identificator)
      css = ['item']
      css << 'active' if active_link?(identificator)
      css.join(' ')
    end

    def link_to_entity(klass_or_entity, action = nil, options = {})
      entity_class = klass_or_entity.is_a?(ActiveRecord::Base) ? klass_or_entity.class : klass_or_entity
      entity = klass_or_entity.is_a?(ActiveRecord::Base) ? klass_or_entity : nil
      options, action = action, nil if action.is_a?(Hash) && action.extractable_options?

      action ||= (entity ? :show : :index)

      if options[:css_item]
        css = css_active_menu_item(entity_class)
        options[:class] ||= ''
        options[:class] << (' ' + css) if css.present?

        options[:no_icon] = true if !options.key?(:no_icon)
      end

      case action
      when :index
        # if @entity && entity_class.parent_entity_class == @entity.class && @parent_entity.nil?
        #   route_array = entity_class.class_route_array(@entity)
        # else
        #   route_array = entity_class.class_route_array(@parent_entity)
        # end

        link_to(polymorphic_path(entity_class.class_route_array(@parent_entity)), class: options[:class], title: entity_class.title_index, data: options[:data]) do
          content_tag(:i, '', class: (options[:no_icon] ? '' : 'icon ' + entity_class.css_class)) + entity_class.button_index
        end
      when :show
        link_to(polymorphic_path(entity.model_route_array), class: options[:class], title: entity.title_show, data: options[:data]) do
          content_tag(:i, '', class: (options[:no_icon] ? '' : 'info icon')) + (options[:no_label] ? '' : entity.button_show)
        end
      when :new
        link_to(new_polymorphic_path(entity_class.class_route_array(@parent_entity)), class: options[:class], title: entity_class.title_new, data: options[:data]) do
          content_tag(:i, '', class: (options[:no_icon] ? '' : 'add icon')) + entity_class.button_new
        end
      when :edit
        link_to(edit_polymorphic_path(entity.model_route_array), class: options[:class], title: entity.title_edit, data: options[:data]) do
          content_tag(:i, '', class: (options[:no_icon] ? '' : 'edit icon')) + (options[:no_label] ? '' : entity.button_edit)
        end
      when :delete
        link_to(polymorphic_path(entity.model_route_array), class: options[:class], title: entity.title_delete, data: {confirm: t(:text_are_you_sure), method: :delete}) do
          content_tag(:i, '', class: (options[:no_icon] ? '' : 'trash icon'))
        end
      end
    end

    def link_to_function(name, function = nil, html_options = {}, &block)
      if block_given?
        content_tag(:a, {href: '#', onclick: "#{name}; return false;"}.merge(function || {}), &block)
      else
        content_tag(:a, name, {href: '#', onclick: "#{function}; return false;"}.merge(html_options))
      end
    end

    def markdown(text)
      options = {
        filter_html: true,
        hard_wrap: true,
        link_attributes: {rel: 'nofollow', target: '_blank'},
        space_after_headers: true,
        fenced_code_blocks: true
      }

      extensions = {
        autolink: true,
        superscript: true,
        disable_indented_code_blocks: true
      }

      renderer = Redcarpet::Render::HTML.new(options)
      markdown = Redcarpet::Markdown.new(renderer, extensions)

      markdown.render(text.to_s).html_safe
    end

    def show_top_menu_new?
      controller.respond_to?(:entity_class) && controller.entity_class && !controller.entity_class.hide_top_menu_new
    end

    def show_top_menu_search?
      controller.respond_to?(:entity_class) && controller.entity_class && !controller.entity_class.hide_top_menu_search
    end

    def home_link(namespace = nil, options = {})
      options = namespace if namespace.is_a?(Hash)

      if namespace == :admin
        link_to(t(:button_administration), main_app.admin_path, {title: t(:title_administration), class: css_active_menu_item(:administration), data: {turbolinks: false}}.merge(options))
      else
        link_to(t(:button_home), main_app.root_path, {title: t(:title_home), class: css_active_menu_item(:home), data: {turbolinks: false}}.merge(options))
      end
    end

    def login_link(options = {})
      css = ['item']
      css << 'active' if controller_name == 'UsersController'

      link_to(t(:button_log_in), main_app.new_user_session_path, {title: t(:title_log_in), class: css.join(' '), data: {turbolinks: false}}.merge(options))
    end

    def sso_login_link(options = {})
      css = ['item']

      link_to(t(:button_log_in), main_app.user_sso_omniauth_authorize_path, {title: t(:title_log_in), class: css.join(' '), data: {turbolinks: false}}.merge(options))
    end

    def signup_link(options = {})
      css = ['item']

      link_to(t(:button_sign_up), 'https://www.easysoftware.com', {title: t(:title_sign_up), class: css.join(' '), data: {turbolinks: false}}.merge(options))
    end

    def logout_link(options = {})
      css = ['item']

      link_to(t(:button_log_out), main_app.destroy_user_session_path, {title: t(:title_log_out), class: css.join(' '), method: :delete, data: {turbolinks: false}}.merge(options))
    end

  end
end
