module Microservice
  module Concerns

    # Example of migration:
    #
    #   create_table :table_name, force: true do |t|
    #     t.string :name, null: false
    #     t.text :description, null: true
    #     t.boolean :active, null: false, default: true
    #
    #     t.references :author, null: false
    #     t.references :updated_by, null: true
    #     t.timestamps null: false
    #  end
    #

    module EntityModel
      extend ActiveSupport::Concern

      included do

        class_attribute :namespace, :search_fields, :sort_field, :parent_entity_symbol
        class_attribute :hide_top_menu_new, :hide_top_menu_search

        scope :sorted, -> {order(sort_field || :name)}
        scope :search, -> (text) {where(Array(search_fields || default_search_fields).map {|f| arel_table[f].matches("%#{text}%")}.inject(:or))}
        scope :active, -> {where(active: true) if column_names.include?('active')}

        belongs_to :author, optional: true, class_name: 'User'
        belongs_to :updated_by, optional: true, class_name: 'User'

      end

      class_methods do

        def belongs_to_parent(name, options={})
          self.parent_entity_symbol = name
          belongs_to(name, options={})
        end

        def default_search_fields
          f = []
          f << :name if self.column_names.include?('name')
          f << :description if self.column_names.include?('description')
          f
        end

        def css_class(_options = {})
          ''
        end

        def description
          I18n.t(self.model_name.i18n_key, scope: [:text_entity_description])
        end

        def parent_entity_id_symbol
          @parent_entity_id_symbol ||= (parent_entity_symbol.to_s + '_id').to_sym if parent_entity_symbol
        end

        def parent_entity_class
          @parent_entity_class ||= self.reflect_on_association(parent_entity_symbol).klass if parent_entity_symbol
        end

        def class_route_array(parent_entity = nil)
          if parent_entity_symbol
            [parent_entity, self.namespace, self.base_class]
          else
            [self.namespace, self.base_class]
          end
        end

        def _to_form_path #:nodoc:
          @_to_form_path ||= begin
            n = ''
            n = "#{self.namespace}/" if self.namespace
            collection = ActiveSupport::Inflector.tableize(name)

            "#{n}#{collection}/form".freeze
          end
        end

      end

      def css_class(options = {})
        self.class.css_class(options)
      end

      def parent_entity
        send(parent_entity_symbol) if parent_entity_symbol
      end

      def parent_entity=(value)
        send(parent_entity_symbol.to_s + '=', value) if parent_entity_symbol
      end

      def polymorphic_class
        self.becomes(self.class.base_class)
      end

      def model_route_array
        [self.namespace, self.parent_entity, polymorphic_class]
      end

      def class_route_array
        self.class.class_route_array
      end

      def to_form_path
        self.class._to_form_path
      end

      def last_touch_by
        updated_by || author
      end

    end
  end
end
